// genss.v
// David Rowe 2 Feb 2009
//
// Generates not SS signal for AVR SPI port. 

module genss (rst, clk, fs, ss);
input        rst;
input        clk;           // 2.048MHz system clock
input        fs;            // TDM bus frame clock
output       ss;            // SS signal to AVR SPI port

reg          state;
reg  [2:0]   nbits;         // bits received so far
reg          ss_b;

parameter IDLE   = 1'b0;    // waiting for next FS
parameter COUNT  = 1'b1;    // SS asserted

always @(negedge clk)
begin
  if (rst) begin
      state       <= IDLE;
      ss_b        <= 0;
  end else begin

    case(state)

 	 IDLE: begin
              if (fs) begin
				    state <= #1 COUNT;
				    nbits <= 0;
					 ss_b  <= 1;
				  end
			  end

    COUNT: begin
              if (nbits == 7) begin
                state  <= #1 IDLE;
                ss_b   <= 0;
              end
              else begin
                nbits <= nbits + 1;
              end 
            end
   
    default: state <= IDLE;

    endcase
  end

end
  
assign ss = ! (fs || ss_b); 

endmodule
