module txuart_tb;
reg        rst;
reg        clk;
reg        tx;
wire       finishedd;
wire       wr;
reg  [7:0] txdata, rxdata, data, pass;

initial begin
  $dumpfile ("txuart_tb.vcd");
  $dumpvars (1, my_txuart);

  // output checked with gtkwave

  //$monitor ("clk=%b, tx=%b, finished=%b wr=%b dout=%b", clk, tx, finished, wr, dout);

  pass = 0;
  tx = 1;
  rst = 1;
  #1 clk = 1;
  #1 clk = 0;
  rst = 0;
  #1 clk = 1;
  #1 clk = 0;

  //----- Send a 10 bit RS232 Word ----------------*/

  // send start bit

  tx = 0;
  // Note: each 155,000 baud RS232 bit is 18 2.048MHz clk cycles 
  repeat (18) begin 
    #1  clk = 1;
    #1  clk = 0;  
  end

  // send data

  txdata = 8'b11010010;
  data = txdata;
  rxdata = 0;
  repeat (8) begin
    tx = data[7];
    repeat (18) begin
      #1  clk = 1;
      if (wr) begin
        rxdata = { rxdata[6:0], tx };	
      end
      #1  clk = 0;  
    end
    data = { data[6:0], 1'b0 };
  end
  if (txdata != rxdata) begin
	$display("TEST FAILED: txdata: %b  rxdata: %b", txdata, rxdata);
  end
  else begin
      pass = pass + 1;
  end

  // send stop bit
  tx = 1;
  repeat (18) begin
    #1  clk = 1;
    #1  clk = 0;  
  end

  //----- Send a 10 bit RS232 Word ----------------*/

  // send start bit

  tx = 0;
  // Note: each 155,000 baud RS232 bit is 18 2.048MHz clk cycles 
  repeat (18) begin 
    #1  clk = 1;
    #1  clk = 0;  
  end

  // send data

  txdata = 8'b10010010;
  data = txdata;
  rxdata = 0;
  repeat (8) begin
    tx = data[7];
    repeat (18) begin
      #1  clk = 1;
      if (wr) begin
        rxdata = { rxdata[6:0], tx };	
      end
      #1  clk = 0;  
    end
    data = { data[6:0], 1'b0 };
  end
  if (txdata != rxdata) begin
	$display("TEST FAILED: txdata: %b  rxdata: %b", txdata, rxdata);
  end
  else begin
      pass = pass + 1;
  end

  // send stop bit
  tx = 1;
  repeat (18) begin
    #1  clk = 1;
    #1  clk = 0;  
  end

 if (pass == 2) begin
    $display("TEST PASSED");
 end
end

txuart my_txuart(
.rst      (rst),
.clk      (clk),
.tx       (tx),
.finished (finished),
.wr       (wr)
);

endmodule

